----------------------------------------------------------------------------------
-- Top module: UART transmitter
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity UART_controller is
    port (
        CLK100MHZ : in STD_LOGIC;
        CPU_RESETN : in STD_LOGIC;
        BTNC : in STD_LOGIC;
        UART_RXD_OUT : out STD_LOGIC);
end UART_controller;

architecture Behavioral of UART_controller is
    constant baud_rate_limit : integer := 100e6 / 9600 - 1; -- 10415 --> 14 bits required
    
    signal clock, reset, prescaler_reset, start : std_logic;
    signal psc_reset, write_data, shr_en, tx_clk : std_logic;
    signal BTNC_old : std_logic;
begin

    clock <= CLK100MHZ;
    reset <= not CPU_RESETN;
    prescaler_reset <= reset or psc_reset;
    
    
    -- sync BTNC and detect rising edge
    process(clock)
    begin 
        if rising_edge(clock) then 
            if reset = '1' then 
                start <= '0';
                BTNC_old <= '0';
            else 
                start <= not BTNC_old and BTNC;
                BTNC_old <= BTNC;
            end if;        
        end if;
    end process;
    
    -- direct instantiation
    baud_rate_gen: entity work.prescaler(Behavioral) 
    generic map (width => 14)
    port map (
        clock => clock,
        reset => prescaler_reset, 
        limit => baud_rate_limit, 
        ce    => tx_clk
    );
    
    PISO: entity work.shift_register(Behavioral) 
    port map(
        clock => clock,
        reset => reset, 
        write_data => write_data, 
        shr_en => shr_en, 
        data_in => "01100001", -- 'a' = 0x61
        data_out => UART_RXD_OUT
    );
    
    controller: entity work.tx_control(Behavioral)
    port map(
        clock => clock,
        reset => reset, 
        write_data => write_data, 
        shr_en => shr_en, 
        psc_reset => psc_reset,
        tx_clk => tx_clk,
        start => start
    );

end Behavioral;
